/* multitalk_curs.c
 * Client Multitalk utilisant les curses pour un affichage graphique.
 * 
 * Auteur: VOURIOT Thierry
 * Web: http://thierry.vouriot.free.fr, http://www.yeri.fr.st
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Date: 10 novembre 2002
 */

#include <curses.h>
#include <ctype.h>

/*------------------------------------------------------------------
 *
 * creation_fenetre() : cree une fenetre de dialogue avec un cadre 
 *            sur n lignes debutant a la ligne d et comportant
 *            le texte t centre sur le bord superieur du cadre
 *            La fenetre occupe toutes les colonnes disponibles (COLS)
 * ENTREE:
 *        n:     nombre de lignes du cadre
 *        d:     ligne debut du cadre
 *        t:     texte affiche dans le haut du cadre
 * RESULTAT:
 *        un pointeur sur la fenetre de dialogue creee
 *
 * REMARQUES:
 *    - Afin de dessiner un cadre, une fenetre     speciale (cadre) 
 *      est creee et la fonction box() est appelee pour cette fenetre.
 *    - La fenetre de dialogue (w) est creee a l'interieur du cadre.
 *    - La fonction newwin() admet 4 parametres :
 *        . le nombre de lignes et le nombre de colonnes
 *        . la ligne debut et la colonne debut
 *-------------------------------------------------------------------*/

WINDOW *creation_fenetre(int n,int d,char *t)
{
    WINDOW *cadre ;            /* la fenetre pour le cadre */
    WINDOW *w ;            /* la fenetre de dialogue */
    
    /* creation du cadre */

    cadre= newwin(n,COLS,d,0) ;
    box(cadre,0,0) ;
    mvwprintw(cadre,0,COLS/2-strlen(t)/2,t) ;
    wrefresh(cadre) ;
    
    /* creation de la fenetre de dialogue */

    w= newwin(n-2,COLS-2,d+1,1) ;
    idlok(w,TRUE) ;
    scrollok(w,TRUE) ;       /* mise en place du defilement (scrolling) */
    wclear(w) ;
    wrefresh(w) ;
    return w ;
}

void win_print(char *s,int n, WINDOW *w) 
{
	int i;
	for (i=0; i<n; i++) {
		if (s[i]!='\0')
		waddch(w,s[i]) ;
	}
        wrefresh(w) ;
} 	

/*-----------------------------------------------------------------------*/

void strchcat(char *s, char c)
{
	int len = strlen(s);
	s[len] = c;
	s[len+1] = '\0';
}


#include "../include/multitalk.h"

/* Prototypes */
void error_quit(char *s);

int main(int argc, char **argv)
{
	WINDOW *f_haut, *f_bas ;    /* les 2 fenetres de dialogue */	
	
	struct hostent *host;
	struct sockaddr_in addr;	// adresse du serveur
	int sock;			// socket
	int port;			// port du serveur
	int nb;				// nb de char envoys
	
	char *message = MALLOCN(char, MSG_SIZE);	// buffer message
	char *message2 = MALLOCN(char, MSG_SIZE);	// buffer message
	int c;
	
	fd_set rfds, afds;	// table des sockets pour la primitive select

	initscr() ;           /* initialisation (obligatoire) de curses */
    	noecho() ;            /* suppression de l'echo des caracteres tapes*/
    	cbreak() ;            /* lecture non bufferisee */

	/* creation des cadres et des fenetres de dialogue */

    	f_haut= creation_fenetre(LINES*0.8,0,"MULTITALK_CURS") ;
    	f_bas = creation_fenetre(LINES*0.2,LINES*0.8,"") ;	
	
	/* Test des arguments de la ligne de commande */
	if (argc != 3) {
		endwin() ; 
		printf("Usage: multitalk nommachine port\n");
		exit(0);
	}
	
	/* Rcupre l'adresse du serveur  partir de son nom */
	host = gethostbyname(argv[1]);
	if (host == NULL) error_quit("error gethostbyname");
	
	/* Cration du socket */
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) error_quit("error socket");
	
	/* Test de la validit du port */
	port = atoi(argv[2]);
	if (port < 1024 || port > 65535) {
		endwin() ; 
		printf("Numero de port invalide: 1024 < port < 65535\n");
		exit(-1);
	}
	
	/* Cration de la structure sockaddr_in (adresse du serveur) */
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);	// port
	memcpy(&addr.sin_addr, host->h_addr, sizeof(addr.sin_addr));	// copie de l'adresse IP
	memset(&(addr.sin_zero), '\0', 8);	// on met  zro le reste de la struct
	
	/* Connection au serveur */
	if (connect(sock, (struct sockaddr*) &addr, sizeof(struct sockaddr)) < 0) 
		error_quit("error connect");
	
	/* Initialisation de la table des sockets */
	FD_ZERO(&afds);
	FD_SET(sock, &afds);
	FD_SET(STDIN, &afds);	
	
	/* Boucle du client */
	while(1) {
		memcpy(&rfds, &afds, sizeof(rfds));	
		
		/* Surveillance des descripteurs en lecture */
		if ( select(sock + 1, &rfds, 0, 0, 0) != -1)
		{	
	
		/* lecture des messages en provenance du serveur
		 * affichage des messages sur la sortie standard
		 */
		if (FD_ISSET(sock, &rfds)) {
			if ((nb = read(sock, message, MSG_SIZE)) < 0) 
				error_quit("error read");
		
			if (nb == 0) { 
				endwin();
				printf("Fermeture de la connexion\n");
				close(sock);
				exit(0);
			}
			win_print(message,nb, f_haut);
		}
		
		/* lecture de l'entre standard
		 * envoi des messages au serveur
		 */
		if (FD_ISSET(STDIN, &rfds)) {	
			c = wgetch(f_bas);
			waddch(f_bas, c);
			wrefresh(f_bas) ;
			strchcat(message2, c);
			
			if (c == '\n') {
			if (write(sock, message2, strlen(message2)) < 0)
				perror("error write");
				message2[0] = '\0';
			}
		}
		}
	}
	
	/* Fermeture du socket */
	close(sock);
	endwin() ; 
	return 1;
}

/* Fonction affichant un message d'erreur en fonction de la valeur
 * de errno et quitte le programme.
 */
void error_quit(char *s)
{
	endwin() ; 
	perror(s);
	exit(-1);
}
